//#include <TinyWireM.h>
//#define Wire TinyWireM
#include "Wire.h"
#include "IRLremote.h"

#define LED_PIN 13 // 1 on Trinket

#ifndef NOT_AN_INTERRUPT
 #define NOT_AN_INTERRUPT -1
#endif
#ifndef digitalPinToInterrupt
 #if (RH_PLATFORM == RH_PLATFORM_ARDUINO) && !defined(__arm__)
 #if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
 // Arduino Mega, Mega ADK, Mega Pro
 // 2->0, 3->1, 21->2, 20->3, 19->4, 18->5
 #define digitalPinToInterrupt(p) ((p) == 2 ? 0 : ((p) == 3 ? 1 : ((p) >= 18 && (p) <= 21 ? 23 - (p) : NOT_AN_INTERRUPT)))
 #elif defined(__AVR_ATmega1284__) || defined(__AVR_ATmega1284P__)
 // Arduino 1284 and 1284P - See Manicbug and Optiboot
 // 10->0, 11->1, 2->2
 #define digitalPinToInterrupt(p) ((p) == 10 ? 0 : ((p) == 11 ? 1 : ((p) == 2 ? 2 : NOT_AN_INTERRUPT)))
 #elif defined(__AVR_ATmega32U4__)
 // Leonardo, Yun, Micro, Pro Micro, Flora, Esplora
 // 3->0, 2->1, 0->2, 1->3, 7->4
 #define digitalPinToInterrupt(p) ((p) == 0 ? 2 : ((p) == 1 ? 3 : ((p) == 2 ? 1 : ((p) == 3 ? 0 : ((p) == 7 ? 4 : NOT_AN_INTERRUPT)))))
 #else
 // All other arduino except Due:
 // Serial Arduino, Extreme, NG, BT, Uno, Diecimila, Duemilanove, Nano, Menta, Pro, Mini 04, Fio, LilyPad, Ethernet etc
 // 2->0, 3->1
 #define digitalPinToInterrupt(p) ((p) == 2 ? 0 : ((p) == 3 ? 1 : NOT_AN_INTERRUPT))
 #endif
 #elif (RH_PLATFORM == RH_PLATFORM_UNO32)
 #define digitalPinToInterrupt(p) ((p) == 38 ? 0 : ((p) == 2 ? 1 : ((p) == 7 ? 2 : ((p) == 8 ? 3 : ((p) == 735 ? 4 : NOT_AN_INTERRUPT)))))
 #else
 // Everything else (including Due and Teensy) interrupt number the same as the interrupt pin number
 #define digitalPinToInterrupt(p) (p)
#endif
#endif

void setup() {
  int pinReceiveIR = digitalPinToInterrupt(3);

  Wire.begin(); // Start I2C Bus as Master
  pinMode(LED_PIN, OUTPUT);
  IRLbegin<IR_ALL>(pinReceiveIR);
}

void loop() {
  if (IRLavailable()) {
    uint8_t protocol;
    uint16_t address, c1, c2;
    uint32_t command;

    digitalWrite(LED_PIN, HIGH);

    protocol = IRLgetProtocol();
    address = IRLgetAddress();
    command = IRLgetCommand();
    c1 = command & 0xFFFF; // lower 16 bits
    c2 = command >> 16; // higher 16 bits

    Wire.beginTransmission(9); // transmit to device #9
    Wire.write(protocol);
    Wire.write(highByte(address));
    Wire.write(lowByte(address));
    Wire.write(highByte(c1));
    Wire.write(lowByte(c1));
    Wire.write(highByte(c2));
    Wire.write(lowByte(c2));
    Wire.endTransmission(); // stop transmitting

    delay(100);
    digitalWrite(LED_PIN, LOW);
    IRLreset();
  }
  delay(50);
}

