#include "Arduino.h" // for millis()
#include "display.h"
#include "ledmatrix.h"

Display::Display() {
  upper = 0;
  lower = 0;
  max_upper = 0;
  max_lower = 0;
  lower_active = false;
  action = 0;
  returnToDispUpper = 0;

  LEDMatrix matrix = LEDMatrix();
}

void Display::begin() {
  matrix.begin(0x70);
  setBrightness(5);
}

void Display::setMaxValues(uint8_t _max_upper, uint8_t _max_lower) {
  max_upper = _max_upper;
  max_lower = _max_lower;
}

void Display::setValues(uint8_t _upper, uint8_t _lower) {
  upper = _upper;
  lower = _lower;
  display();
}

void Display::activateLower(uint8_t seconds) {
  lower_active = true;
  returnToDispUpper = millis() + seconds * 1000;
  display();
}

void Display::setBrightness(uint8_t b) {
  matrix.setBrightness(b);
}

void Display::setBlinkRate(uint8_t b) {
  matrix.setBlinkRate(b);
}

void Display::setAction(uint8_t b) {
  action = b;
  display();
}

void Display::step() {
  if (returnToDispUpper > 0) {
    if (millis() > returnToDispUpper) {
      lower_active = false;
      returnToDispUpper = 0;
      display();
    }
  }
}

void Display::display() {
  char buffer[4];
  int16_t y_text = 0;
  int16_t y_line = 6;
  uint8_t value = upper;
  int16_t percent_upper, percent_lower, n;

  percent_upper = (uint8_t) (8. * upper / max_upper);
  percent_lower = (uint8_t) (8. * lower / max_lower);

  matrix.clear();
  if (lower_active) {
    y_text = 3;
    y_line = 0;
    value = lower;
  }

  if (value > 99) {
    matrix.writeText(0, y_text, "XX");
  } else {
    if (value > 9) {
      matrix.drawChar(0, y_text, 0x30+value/10);
    }
    matrix.drawChar(4, y_text, 0x30+value%10);

    for (n = 0; n < percent_upper; n++) {
      matrix.drawPixel(n, y_line, LED_ON);
    }
    y_line++;
    for (n = 0; n < percent_lower; n++) {
      matrix.drawPixel(n, y_line, LED_ON);
    }
  }
  if (action) {
    matrix.drawPixel(0, y_text, LED_ON);
  }
  matrix.writeDisplay();
}

