#include <Wire.h>
#include <avr/pgmspace.h>
#include "ledmatrix.h"
#include "font4x6.h"

LEDMatrix::LEDMatrix(void) {
}

void LEDMatrix::begin(uint8_t _addr = 0x70) {
  i2c_addr = _addr;

  Wire.begin();
  Wire.beginTransmission(i2c_addr);
  Wire.write(0x20 | 0x1);  // Turn on System oscillator (normal operation mode)
  Wire.endTransmission();
  setBrightness(15); // max brightness
  setBlinkRate(BLINK_OFF);
}

void LEDMatrix::writeDisplay(void) {
  Wire.beginTransmission(i2c_addr);
  Wire.write(0x00); // start at address $00

  for (uint8_t i=0; i<8; i++) {
    Wire.write(displaybuffer[i]);
    Wire.write(0);
  }
  Wire.endTransmission();
}

void LEDMatrix::clear(void) {
  memset(displaybuffer, 0, 8*sizeof(uint8_t));
}

void LEDMatrix::setBrightness(uint8_t b) {
  if (b > 15) {
    b = 15;
  }
  Wire.beginTransmission(i2c_addr);
  Wire.write(0xE0 | b); // Dimming set
  Wire.endTransmission();
}

void LEDMatrix::setBlinkRate(uint8_t b) {
  if (b > 3 || b < 0) {
    b = 0;
  }
  
  Wire.beginTransmission(i2c_addr);
  Wire.write(0x80 | 0x01 | (b << 1)); // Display setup, always display on
  Wire.endTransmission();
}

void LEDMatrix::drawPixel(int8_t x, int8_t y, uint8_t on) {
  int16_t t;

  if ((y < 0) || (y >= 8)) {
    return;
  }
  if ((x < 0) || (x >= 8)) {
    return;
  }

  // the Adafruit Mini 8x8 LED Matrix w/I2C Backpack doesn't need these two lines,
  // but the Adafruit Small 1.2" 8x8 LED Matrix w/I2C Backpack does
  t = x;
  x = y;
  y = t;
  y = 8 - y - 1;

  x += 7;
  x %= 8;

  if (on) {
    displaybuffer[y] |= 1 << x;
  } else {
    displaybuffer[y] &= ~(1 << x);
  }
}

void LEDMatrix::drawChar(int8_t x, int8_t y, unsigned char c) {
  if((x >= 8)       || // Clip right
     (y >= 8)       || // Clip bottom
     ((x + 4) < 0)  || // Clip left
     ((y + 6) < 0))    // Clip top
  {
    return;
  }

  for (int8_t i = 0; i < 6; i++ ) {
    uint8_t line = pgm_read_byte(font4x6+(c*6)+i);
    line >>= 4;
    for (int8_t j = 0; j < 4; j++) {
      if (line & 0x1) {
        drawPixel(x+j, y+i, LED_ON);
      }
      line >>= 1;
    }
  }
}

void LEDMatrix::writeText(int8_t x, int8_t y, const char *text) {
  int16_t textlen = strlen(text);
  for (int8_t i = 0; i < textlen; i++) {
    drawChar(x+4*i, y, text[i]);
  }
}

