#include "ripple.h"

Ripple::Ripple(uint16_t nl) {
  num_leds = nl;
  center = 0;
  step = -1;
  maxSteps = 16;
  fadeRate = 0.8;

  currentBg = random(256);
  nextBg = currentBg;
}

void Ripple::nextFrame(CRGB *leds) {
  if (currentBg == nextBg) {
    nextBg = random(256);
  } else if (nextBg > currentBg) {
    currentBg++;
  } else {
    currentBg--;
  }
  for (uint16_t l = 0; l < num_leds; l++) {
    leds[l] = CHSV(currentBg, 255, 50);
  }

  if (step == -1) {
    center = random(num_leds);
    color = random(256);
    step = 0;
  }

  if (step == 0) {
    leds[center] = CHSV(color, 255, 255);
    step++;
  } else {
    if (step < maxSteps) {
      leds[wrap(center + step)] = CHSV(color, 255, pow(fadeRate, step) * 255);
      leds[wrap(center - step)] = CHSV(color, 255, pow(fadeRate, step) * 255);
      if (step > 3) {
        leds[wrap(center + step - 3)] = CHSV(color, 255, pow(fadeRate, step - 2) * 255);
        leds[wrap(center - step + 3)] = CHSV(color, 255, pow(fadeRate, step - 2) * 255);
      }
      step ++;
    } else {
      step = -1;
    }
  }
  FastLED.delay(40);
}

int Ripple::wrap(int step) {
  if (step < 0) {
    return num_leds + step;
  }
  if (step > num_leds - 1) {
    return step - num_leds;
  }
  return step;
}

